/*

Fenryxo's library :-)

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

If this file is built as a shared library it's released under GNU LGPL 3+ and 2-Clause BSD.
Otherwise it's released under GNU GPL 3+.
 
This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See <http://www.gnu.org/licenses/> for more details.
*/

using Fenryxo.Logger;
namespace Fenryxo{
	public errordomain AppError{
		NOT_RUNNING,
		ALREADY_RUNNING,
		UNABLE_TO_TERMINATE,
		UNABLE_TO_ACTIVATE
	}
	
	public abstract class Application : Unique.App{
		private bool remote = true;
		
		public string desktop_entry{ get; protected set;}
		public string app_name{ get; protected set;}
		public string path_name{ get; protected set;}
		public string display_name{ get; protected set;}
		public string icon{ get; protected set; default = "";}
		public string version{ get; protected set; default = "";}
		
		
		public Application(string name, string desktop_entry, string path_name,
		                   string? startup_id=null){
			Object(name: name, startup_id: startup_id ?? name);
			this.display_name = name;
			this.app_name = name;
			this.desktop_entry = desktop_entry;
			this.path_name = path_name;
		}
		
		public virtual signal void terminated(){}
		public virtual signal void activated(){}
		public virtual signal void launched(){}
		
		public override Unique.Response message_received(int command,
			Unique.MessageData message_data, uint time){
			if(command == Unique.Command.CLOSE){
				lib_message("Unique: Current instance will quit.");
				this.terminated();
				return Unique.Response.OK;
			}
			if(command == Unique.Command.ACTIVATE){
				lib_message("Unique: Current instance will raise.");
				this.activated();
				return Unique.Response.OK;
			}
			return Unique.Response.INVALID;
		} 
		
		public void quit() throws AppError{
			if(!this.remote){
				this.terminated();
				return;
			}
			if(!this.is_running){
				throw new AppError.NOT_RUNNING("Application @name not running");
			}

			lib_message("Application: Old instance will be terminated.");
			if(this.send_message(Unique.Command.CLOSE, null) != Unique.Response.OK){
				throw new AppError.UNABLE_TO_TERMINATE("Old instance termination failed!");
			}
		}
		
		public bool activate() throws AppError{
			if(!this.remote){
				this.activated();
				return true;
			}
			else if(this.is_running){
				if(this.send_message(Unique.Command.ACTIVATE, null) != Unique.Response.OK){
					throw new AppError.UNABLE_TO_ACTIVATE("Old instance activation failed!");
				}
				return true;
			}
			else{
				return false;
			}
		}

		
		public bool start() throws AppError{
			if(this.activate()){
				return false;
			}
			else{
				this.run();
				this.launched();
				this.remote = false;
				return true;
			}
		}
		
		protected abstract void run();
	}
}
